<?php

namespace CC\CryptoCloud\Payment;

use Exception;
use GuzzleHttp\Exception\GuzzleException;
use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Http\Request;
use XF\Mvc\Controller;
use XF\Mvc\Reply\AbstractReply;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;

class CryptoCloud extends AbstractProvider
{
	/**
	 * @return string
	 */
	public function getTitle(): string
	{
		return 'CryptoCloud';
	}

	/**
	 * @return string
	 */
	public function getApiEndpoint(): string
	{
		return 'https://api.cryptocloud.plus/v1/';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 *
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = []): bool
	{
		if (empty($options['shop_id']) || empty($options['api_key']) || empty($options['secret_key']))
		{
			$errors[] = XF::phrase('cc_you_must_provide_all_data');

			return false;
		}

		return true;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase): array
	{
		return [
			'shop_id'  => $purchaseRequest->PaymentProfile->options['shop_id'],
			'amount'   => $purchase->cost,
			'order_id' => $purchaseRequest->request_key,
			'currency' => $purchase->currency,
			'email'    => $purchaseRequest->extra_data['email'] ?? $purchase->purchaser->email
		];
	}

	/**
	 * @param Controller      $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return AbstractReply
	 * @throws GuzzleException
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase): XF\Mvc\Reply\AbstractReply
	{
		$profileOptions = $purchase->paymentProfile->options;
		$payment = $this->getPaymentParams($purchaseRequest, $purchase);

		try
		{
			$response = XF::app()->http()->client()->post($this->getApiEndpoint() . 'invoice/create', [
				'headers'     => [
					'Authorization' => 'Token ' . $profileOptions['api_key']
				],
				'form_params' => $payment
			]);
		}
		catch (Exception $e)
		{
			XF::logException($e);

			return $controller->error(XF::phrase('something_went_wrong_please_try_again'));
		}

		if ($response)
		{
			$responseData = json_decode($response->getBody()->getContents(), true);

			if (!empty($responseData['pay_url']))
			{
				return $controller->redirect($responseData['pay_url']);
			}
		}

		return $controller->error(XF::phrase('something_went_wrong_please_try_again'));
	}

	/**
	 * @param Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(Request $request): CallbackState
	{
		$state = new CallbackState();

		$state->providerId = $request->filter('_xfProvider', 'str');

		$state->input = $request->filter([
			'status'        => 'str',
			'invoice_id'    => 'str',
			'amount_crypto' => 'str',
			'currency'      => 'str',
			'order_id'      => 'str',
			'token'         => 'str'
		]);

		$state->transactionId = $state->input['invoice_id'] ?? null;
		$state->requestKey = $state->input['order_id'] ?? null;

		$state->token = $state->input['token'] ?? null;
		$state->status = $state->input['status'] ?? null;

		$state->ip = $request->getIp();
		$state->rawInput = $request->getInputForLogs();

		$state->httpCode = 200;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateCallback(CallbackState $state): bool
	{
		if ($state->transactionId && $state->requestKey)
		{
			if ($state->getPaymentProfile()->provider_id == $state->providerId)
			{
				return parent::validateCallback($state);
			}

			$state->logType = 'info';
			$state->logMessage = 'Invalid provider.';

			return false;
		}

		$state->logType = 'info';
		$state->logMessage = 'No Transaction ID or Request Key. No action to take.';

		return false;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validatePurchasableData(CallbackState $state): bool
	{
		$paymentProfile = $state->getPaymentProfile();

		$options = $paymentProfile->options;
		if (!empty($options['secret_key']))
		{
			if (!empty($state->token))
			{
				$jwtParts = explode('.', $state->token);
				$signature = $jwtParts[2];

				$generatedSignature = hash_hmac('sha256', $jwtParts[0] . '.' . $jwtParts[1], $options['secret_key'], true);
				$generatedSignature = strtr(rtrim(base64_encode($generatedSignature), '='), '+/', '-_');

				if (hash_equals($signature, $generatedSignature))
				{
					return true;
				}

				$state->logType = 'error';
				$state->logMessage = 'Invalid signature.';

				return false;
			}

			$state->logType = 'error';
			$state->logMessage = 'Empty signature.';

			return false;
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid secret key.';

		return false;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state): void
	{
		if (strtolower($state->status) == 'success')
		{
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		}
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state): void
	{
		$state->logDetails = [
			'ip'           => $state->ip,
			'request_time' => XF::$time,
			'raw_input'    => $state->rawInput,
		];
	}

	/**
	 * @var array
	 */
	protected $supportedCurrencies = [
		'USD', 'EUR', 'GBP', 'RUB'
	];

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $unit
	 * @param                $amount
	 * @param int            $result
	 *
	 * @return bool
	 */
	public function supportsRecurring(PaymentProfile $paymentProfile, $unit, $amount, &$result = self::ERR_NO_RECURRING): bool
	{
		return false;
	}

	/**
	 * @return array
	 */
	protected function getSupportedRecurrenceRanges(): array
	{
		return [];
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $currencyCode
	 *
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode): bool
	{
		return in_array($currencyCode, $this->supportedCurrencies);
	}
}
